@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -36,7 +36,6 @@ struct connman_network {
 	struct connman_element element;
 	enum connman_network_type type;
 	enum connman_network_protocol protocol;
-	connman_bool_t secondary;
 	connman_bool_t available;
 	connman_bool_t connected;
 	connman_bool_t roaming;
@@ -66,6 +65,13 @@ struct connman_network {
 		unsigned short channel;
 		char *security;
 		char *passphrase;
+		char *eap;
+		char *identity;
+		char *ca_cert_path;
+		char *client_cert_path;
+		char *private_key_path;
+		char *private_key_passphrase;
+		char *phase2_auth;
 	} wifi;
 };
 
@@ -75,6 +81,8 @@ static const char *type2string(enum connman_network_type type)
 	case CONNMAN_NETWORK_TYPE_UNKNOWN:
 	case CONNMAN_NETWORK_TYPE_VENDOR:
 		break;
+	case CONNMAN_NETWORK_TYPE_ETHERNET:
+		return "ethernet";
 	case CONNMAN_NETWORK_TYPE_WIFI:
 		return "wifi";
 	case CONNMAN_NETWORK_TYPE_WIMAX:
@@ -83,8 +91,6 @@ static const char *type2string(enum connman_network_type type)
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 		return "bluetooth";
 	case CONNMAN_NETWORK_TYPE_CELLULAR:
-	case CONNMAN_NETWORK_TYPE_MBM:
-	case CONNMAN_NETWORK_TYPE_HSO:
 		return "cellular";
 	}
 
@@ -110,121 +116,71 @@ static DBusMessage *get_properties(DBusConnection *conn,
 
 	dbus_message_iter_init_append(reply, &array);
 
-	dbus_message_iter_open_container(&array, DBUS_TYPE_ARRAY,
-			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
-			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
+	connman_dbus_dict_open(&array, &dict);
 
 	if (network->device) {
 		const char *path = connman_device_get_path(network->device);
 		if (path != NULL)
-			connman_dbus_dict_append_variant(&dict, "Device",
+			connman_dbus_dict_append_basic(&dict, "Device",
 						DBUS_TYPE_OBJECT_PATH, &path);
 	}
 
 	if (network->address != NULL)
-		connman_dbus_dict_append_variant(&dict, "Address",
+		connman_dbus_dict_append_basic(&dict, "Address",
 					DBUS_TYPE_STRING, &network->address);
 
 	if (network->name != NULL)
-		connman_dbus_dict_append_variant(&dict, "Name",
+		connman_dbus_dict_append_basic(&dict, "Name",
 					DBUS_TYPE_STRING, &network->name);
 
-	connman_dbus_dict_append_variant(&dict, "Connected",
+	connman_dbus_dict_append_basic(&dict, "Connected",
 				DBUS_TYPE_BOOLEAN, &network->connected);
 
 	if (network->strength > 0)
-		connman_dbus_dict_append_variant(&dict, "Strength",
+		connman_dbus_dict_append_basic(&dict, "Strength",
 					DBUS_TYPE_BYTE, &network->strength);
 
 	if (network->frequency > 0)
-		connman_dbus_dict_append_variant(&dict, "Frequency",
+		connman_dbus_dict_append_basic(&dict, "Frequency",
 					DBUS_TYPE_UINT16, &network->frequency);
 
 	if (network->wifi.ssid != NULL && network->wifi.ssid_len > 0)
-		connman_dbus_dict_append_array(&dict, "WiFi.SSID",
+		connman_dbus_dict_append_fixed_array(&dict, "WiFi.SSID",
 				DBUS_TYPE_BYTE, &network->wifi.ssid,
 						network->wifi.ssid_len);
 
 	if (network->wifi.mode != NULL)
-		connman_dbus_dict_append_variant(&dict, "WiFi.Mode",
+		connman_dbus_dict_append_basic(&dict, "WiFi.Mode",
 				DBUS_TYPE_STRING, &network->wifi.mode);
 
 	if (network->wifi.channel > 0)
-		connman_dbus_dict_append_variant(&dict, "WiFi.Channel",
+		connman_dbus_dict_append_basic(&dict, "WiFi.Channel",
 				DBUS_TYPE_UINT16, &network->wifi.channel);
 
-	if (network->wifi.security != NULL)
-		connman_dbus_dict_append_variant(&dict, "WiFi.Security",
+	if (network->wifi.security != NULL) {
+		connman_dbus_dict_append_basic(&dict, "WiFi.Security",
 				DBUS_TYPE_STRING, &network->wifi.security);
 
+		if (g_strcmp0(network->wifi.security, "ieee8021x") == 0 &&
+						    network->wifi.eap != NULL)
+			connman_dbus_dict_append_basic(&dict, "WiFi.EAP",
+					DBUS_TYPE_STRING, &network->wifi.eap);
+	}
+
+
 	if (network->wifi.passphrase != NULL &&
 			__connman_security_check_privilege(msg,
 				CONNMAN_SECURITY_PRIVILEGE_SECRET) == 0)
-		connman_dbus_dict_append_variant(&dict, "WiFi.Passphrase",
+		connman_dbus_dict_append_basic(&dict, "WiFi.Passphrase",
 				DBUS_TYPE_STRING, &network->wifi.passphrase);
 
-	dbus_message_iter_close_container(&array, &dict);
+	connman_dbus_dict_close(&array, &dict);
 
 	return reply;
 }
 
-static DBusMessage *set_property(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_network *network = data;
-	DBusMessageIter iter, value;
-	const char *name;
-	int type;
-
-	DBG("conn %p", conn);
-
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
-		return __connman_error_invalid_arguments(msg);
-
-	dbus_message_iter_get_basic(&iter, &name);
-	dbus_message_iter_next(&iter);
-	dbus_message_iter_recurse(&iter, &value);
-
-	if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
-		return __connman_error_permission_denied(msg);
-
-	type = dbus_message_iter_get_arg_type(&value);
-
-	if (g_str_equal(name, "WiFi.Passphrase") == TRUE) {
-		const char *passphrase;
-
-		if (type != DBUS_TYPE_STRING)
-			return __connman_error_invalid_arguments(msg);
-
-		if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_SECRET) < 0)
-			return __connman_error_permission_denied(msg);
-
-		dbus_message_iter_get_basic(&value, &passphrase);
-
-		g_free(network->wifi.passphrase);
-		network->wifi.passphrase = g_strdup(passphrase);
-	} else if (g_str_has_prefix(name, "IPv4.") == TRUE) {
-		int err;
-
-		if (network->ipconfig == NULL)
-			return __connman_error_invalid_property(msg);
-
-		err = __connman_ipconfig_set_ipv4(network->ipconfig,
-							name + 5, &value);
-		if (err < 0)
-			return __connman_error_failed(msg, -err);
-	} else
-		return __connman_error_invalid_property(msg);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
 static GDBusMethodTable network_methods[] = {
 	{ "GetProperties", "",   "a{sv}", get_properties },
-	{ "SetProperty",   "sv", "",      set_property   },
 	{ },
 };
 
@@ -235,43 +191,21 @@ static GDBusSignalTable network_signals[] = {
 
 static DBusConnection *connection;
 
-static void append_networks(struct connman_device *device,
-						DBusMessageIter *entry)
+static void append_networks(DBusMessageIter *iter, void *user_data)
 {
-	DBusMessageIter value, iter;
-	const char *key = "Networks";
+	struct connman_device *device = user_data;
 
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
 	__connman_element_list((struct connman_element *) device,
-					CONNMAN_ELEMENT_TYPE_NETWORK, &iter);
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(entry, &value);
+					CONNMAN_ELEMENT_TYPE_NETWORK, iter);
 }
 
 static void emit_networks_signal(struct connman_device *device)
 {
 	const char *path = connman_device_get_path(device);
-	DBusMessage *signal;
-	DBusMessageIter entry;
 
-	signal = dbus_message_new_signal(path,
-				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	append_networks(device, &entry);
-
-	g_dbus_send_message(connection, signal);
+	connman_dbus_property_changed_array(path,
+			CONNMAN_DEVICE_INTERFACE, "Networks",
+			DBUS_TYPE_OBJECT_PATH, append_networks, device);
 }
 
 static int register_interface(struct connman_element *element)
@@ -303,7 +237,8 @@ static void unregister_interface(struct connman_element *element)
 
 	network->registered = FALSE;
 
-	emit_networks_signal(network->device);
+	if (network->device != NULL)
+		emit_networks_signal(network->device);
 
 	g_dbus_unregister_interface(connection, element->path,
 						CONNMAN_NETWORK_INTERFACE);
@@ -379,6 +314,8 @@ static void network_destruct(struct connman_element *element)
 		connman_ipconfig_unref(network->ipconfig);
 		network->ipconfig = NULL;
 	}
+
+	network->device = NULL;
 }
 
 /**
@@ -432,7 +369,6 @@ struct connman_network *connman_network_create(const char *identifier,
 	connman_element_set_uint8(&network->element, "Strength", strength);
 
 	network->type       = type;
-	network->secondary  = FALSE;
 	network->identifier = g_strdup(temp);
 
 	return network;
@@ -510,6 +446,31 @@ const char *connman_network_get_path(struct connman_network *network)
  */
 void connman_network_set_index(struct connman_network *network, int index)
 {
+	struct connman_service *service;
+
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL)
+		goto done;
+
+	if (network->element.index < 0)
+		/*
+		 * This is needed for plugins that havent set their ipconfig
+		 * layer yet, due to not being able to get a network index
+		 * prior to creating a service.
+		 */
+		__connman_service_create_ipconfig(service, index);
+	else {
+		struct connman_ipconfig *ipconfig;
+
+		/* If index changed, the index of ipconfig must be reset. */
+		ipconfig = __connman_service_get_ipconfig(service);
+		if (ipconfig == NULL)
+			goto done;
+
+		__connman_ipconfig_set_index(ipconfig, index);
+	}
+
+done:
 	network->element.index = index;
 }
 
@@ -524,6 +485,18 @@ int connman_network_get_index(struct connman_network *network)
 	return network->element.index;
 }
 
+/**
+ * connman_network_get_element:
+ * @network: network structure
+ *
+ * Get connman_element of network
+ */
+struct connman_element *connman_network_get_element(
+				struct connman_network *network)
+{
+	return &network->element;
+}
+
 /**
  * connman_network_set_protocol:
  * @network: network structure
@@ -547,18 +520,14 @@ void connman_network_set_protocol(struct connman_network *network,
 void connman_network_set_group(struct connman_network *network,
 							const char *group)
 {
-	if (network->secondary == TRUE)
-		return;
-
 	switch (network->type) {
 	case CONNMAN_NETWORK_TYPE_UNKNOWN:
 	case CONNMAN_NETWORK_TYPE_VENDOR:
 		return;
+	case CONNMAN_NETWORK_TYPE_ETHERNET:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 	case CONNMAN_NETWORK_TYPE_CELLULAR:
-	case CONNMAN_NETWORK_TYPE_MBM:
-	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		break;
@@ -603,20 +572,18 @@ const char *__connman_network_get_ident(struct connman_network *network)
 
 connman_bool_t __connman_network_get_weakness(struct connman_network *network)
 {
-	if (network->secondary == TRUE)
-		return FALSE;
-
 	switch (network->type) {
 	case CONNMAN_NETWORK_TYPE_UNKNOWN:
 	case CONNMAN_NETWORK_TYPE_VENDOR:
+	case CONNMAN_NETWORK_TYPE_ETHERNET:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 	case CONNMAN_NETWORK_TYPE_CELLULAR:
-	case CONNMAN_NETWORK_TYPE_MBM:
-	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		break;
 	case CONNMAN_NETWORK_TYPE_WIFI:
+		if (g_strcmp0(network->wifi.mode, "adhoc") == 0)
+			return TRUE;
 		if (network->strength > 0 && network->strength < 20)
 			return TRUE;
 		break;
@@ -625,6 +592,11 @@ connman_bool_t __connman_network_get_weakness(struct connman_network *network)
 	return FALSE;
 }
 
+connman_bool_t __connman_network_get_connecting(struct connman_network *network)
+{
+	return network->connecting;
+}
+
 /**
  * connman_network_set_available:
  * @network: network structure
@@ -702,61 +674,166 @@ static void set_associate_error(struct connman_network *network)
 					CONNMAN_SERVICE_STATE_FAILURE);
 }
 
+static void set_configure_error(struct connman_network *network)
+{
+	struct connman_service *service;
+
+	network->connecting = FALSE;
+
+	service = __connman_service_lookup_from_network(network);
+
+	__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_FAILURE);
+}
+
+void connman_network_set_method(struct connman_network *network,
+					enum connman_ipconfig_method method)
+{
+	struct connman_service *service;
+	struct connman_ipconfig *ipconfig;
+
+	network->element.ipv4.method = method;
+
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL)
+		return;
+
+	ipconfig = __connman_service_get_ipconfig(service);
+	if (ipconfig == NULL)
+		return;
+
+	connman_ipconfig_set_method(ipconfig, method);
+}
+
 void connman_network_set_error(struct connman_network *network,
 					enum connman_network_error error)
 {
 	DBG("nework %p, error %d", network, error);
 
+	network->connecting = FALSE;
+
 	switch (error) {
 	case CONNMAN_NETWORK_ERROR_UNKNOWN:
 		return;
 	case CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL:
 		set_associate_error(network);
 		break;
+	case CONNMAN_NETWORK_ERROR_CONFIGURE_FAIL:
+		set_configure_error(network);
+		break;
 	}
 }
 
+static void set_configuration(struct connman_network *network)
+{
+	struct connman_service *service;
+
+	DBG("network %p", network);
+
+	__connman_device_increase_connections(network->device);
+
+	__connman_device_set_network(network->device, network);
+
+	connman_device_set_disconnected(network->device, FALSE);
+
+	service = __connman_service_lookup_from_network(network);
+	__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_CONFIGURATION);
+}
+
+static void set_connected_manual(struct connman_network *network)
+{
+	struct connman_service *service;
+	struct connman_ipconfig *ipconfig;
+	int err;
+
+	DBG("network %p", network);
+
+	service = __connman_service_lookup_from_network(network);
+
+	ipconfig = __connman_service_get_ipconfig(service);
+
+	set_configuration(network);
+
+	err = __connman_ipconfig_set_address(ipconfig);
+	if (err < 0) {
+		connman_network_set_error(network,
+			CONNMAN_NETWORK_ERROR_CONFIGURE_FAIL);
+		return;
+	}
+
+	__connman_ipconfig_set_gateway(ipconfig, &network->element);
+
+	network->connecting = FALSE;
+
+	connman_network_set_associating(network, FALSE);
+
+	__connman_service_indicate_state(service, CONNMAN_SERVICE_STATE_READY);
+}
+
+static int set_connected_dhcp(struct connman_network *network)
+{
+	struct connman_element *element;
+	int error;
+
+	DBG("network %p", network);
+
+	if (network->protocol != CONNMAN_NETWORK_PROTOCOL_IP)
+		return -EINVAL;
+
+	element = connman_element_create(NULL);
+	if (element == NULL)
+		return -ENOMEM;
+
+	element->type  = CONNMAN_ELEMENT_TYPE_DHCP;
+	element->index = network->element.index;
+
+	error = connman_element_register(element, &network->element);
+	if (error < 0) {
+		connman_element_unref(element);
+		return error;
+	}
+
+	set_configuration(network);
+
+	return 0;
+}
+
 static gboolean set_connected(gpointer user_data)
 {
 	struct connman_network *network = user_data;
 	struct connman_service *service;
+	struct connman_ipconfig *ipconfig;
+	enum connman_ipconfig_method method;
 
 	service = __connman_service_lookup_from_network(network);
 
+	ipconfig = __connman_service_get_ipconfig(service);
+
+	method = __connman_ipconfig_get_method(ipconfig);
+
+	DBG("method %d", method);
+
 	if (network->connected == TRUE) {
-		struct connman_element *element;
-		enum connman_element_type type = CONNMAN_ELEMENT_TYPE_UNKNOWN;
-
-		switch (network->protocol) {
-		case CONNMAN_NETWORK_PROTOCOL_UNKNOWN:
-			return 0;
-		case CONNMAN_NETWORK_PROTOCOL_IP:
-			type = CONNMAN_ELEMENT_TYPE_DHCP;
-			break;
-		case CONNMAN_NETWORK_PROTOCOL_PPP:
-			type = CONNMAN_ELEMENT_TYPE_PPP;
-			break;
+		switch (method) {
+		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+		case CONNMAN_IPCONFIG_METHOD_OFF:
+			return FALSE;
+		case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		case CONNMAN_IPCONFIG_METHOD_FIXED:
+			set_connected_manual(network);
+			return TRUE;
+		case CONNMAN_IPCONFIG_METHOD_DHCP:
+			if (set_connected_dhcp(network) < 0) {
+				connman_network_set_error(network,
+					CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL);
+				return FALSE;
+			}
 		}
 
-		__connman_device_increase_connections(network->device);
-
-		__connman_device_set_network(network->device, network);
-
-		connman_device_set_disconnected(network->device, FALSE);
-
-		element = connman_element_create(NULL);
-		if (element != NULL) {
-			element->type  = type;
-			element->index = network->element.index;
-
-			if (connman_element_register(element,
-						&network->element) < 0)
-				connman_element_unref(element);
-
-			__connman_service_indicate_state(service,
-					CONNMAN_SERVICE_STATE_CONFIGURATION);
-		}
 	} else {
+		struct connman_service *service;
+
 		connman_element_unregister_children(&network->element);
 
 		__connman_device_set_network(network->device, NULL);
@@ -764,6 +841,8 @@ static gboolean set_connected(gpointer user_data)
 
 		__connman_device_decrease_connections(network->device);
 
+		service = __connman_service_lookup_from_network(network);
+
 		__connman_service_indicate_state(service,
 						CONNMAN_SERVICE_STATE_IDLE);
 	}
@@ -785,10 +864,6 @@ static gboolean set_connected(gpointer user_data)
 int connman_network_set_connected(struct connman_network *network,
 						connman_bool_t connected)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "Connected";
-
 	DBG("network %p connected %d", network, connected);
 
 	if ((network->connecting == TRUE || network->associating == TRUE) &&
@@ -808,21 +883,9 @@ int connman_network_set_connected(struct connman_network *network,
 		return 0;
 	}
 
-	signal = dbus_message_new_signal(network->element.path,
-				CONNMAN_NETWORK_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return 0;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN, &connected);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
+	connman_dbus_property_changed_basic(network->element.path,
+				CONNMAN_NETWORK_INTERFACE, "Connected",
+						DBUS_TYPE_BOOLEAN, &connected);
 
 	set_connected(network);
 
@@ -883,8 +946,10 @@ int __connman_network_connect(struct connman_network *network)
 	if (err < 0) {
 		if (err == -EINPROGRESS)
 			connman_network_set_associating(network, TRUE);
-		else
+		else {
+			network->connecting = FALSE;
 			network->hidden = FALSE;
+		}
 
 		return err;
 	}
@@ -921,7 +986,7 @@ int __connman_network_disconnect(struct connman_network *network)
 
 	err = network->driver->disconnect(network);
 	if (err == 0) {
-		network->connected = FALSE;
+		connman_network_set_connected(network, FALSE);
 		set_connected(network);
 	}
 
@@ -1025,6 +1090,8 @@ int connman_network_set_roaming(struct connman_network *network,
 int connman_network_set_string(struct connman_network *network,
 					const char *key, const char *value)
 {
+	int err;
+
 	DBG("network %p key %s value %s", network, key, value);
 
 	if (g_strcmp0(key, "Name") == 0)
@@ -1045,9 +1112,40 @@ int connman_network_set_string(struct connman_network *network,
 	} else if (g_str_equal(key, "WiFi.Passphrase") == TRUE) {
 		g_free(network->wifi.passphrase);
 		network->wifi.passphrase = g_strdup(value);
+	} else if (g_str_equal(key, "WiFi.EAP") == TRUE) {
+		g_free(network->wifi.eap);
+		network->wifi.eap = g_strdup(value);
+	} else if (g_str_equal(key, "WiFi.Identity") == TRUE) {
+		g_free(network->wifi.identity);
+		network->wifi.identity = g_strdup(value);
+	} else if (g_str_equal(key, "WiFi.CACertFile") == TRUE) {
+		g_free(network->wifi.ca_cert_path);
+		network->wifi.ca_cert_path = g_strdup(value);
+	} else if (g_str_equal(key, "WiFi.ClientCertFile") == TRUE) {
+		g_free(network->wifi.client_cert_path);
+		network->wifi.client_cert_path = g_strdup(value);
+	} else if (g_str_equal(key, "WiFi.PrivateKeyFile") == TRUE) {
+		g_free(network->wifi.private_key_path);
+		network->wifi.private_key_path = g_strdup(value);
+	} else if (g_str_equal(key, "WiFi.PrivateKeyPassphrase") == TRUE) {
+		g_free(network->wifi.private_key_passphrase);
+		network->wifi.private_key_passphrase = g_strdup(value);
+	} else if (g_str_equal(key, "WiFi.Phase2") == TRUE) {
+		g_free(network->wifi.phase2_auth);
+		network->wifi.phase2_auth = g_strdup(value);
 	}
 
-	return connman_element_set_string(&network->element, key, value);
+	err = connman_element_set_string(&network->element, key, value);
+	if (err < 0)
+		return err;
+
+	if (network->driver == NULL)
+		return 0;
+
+	if (network->driver->setup)
+		return network->driver->setup(network, key);
+
+	return 0;
 }
 
 /**
@@ -1074,6 +1172,20 @@ const char *connman_network_get_string(struct connman_network *network,
 		return network->wifi.security;
 	else if (g_str_equal(key, "WiFi.Passphrase") == TRUE)
 		return network->wifi.passphrase;
+	else if (g_str_equal(key, "WiFi.EAP") == TRUE)
+		return network->wifi.eap;
+	else if (g_str_equal(key, "WiFi.Identity") == TRUE)
+		return network->wifi.identity;
+	else if (g_str_equal(key, "WiFi.CACertFile") == TRUE)
+		return network->wifi.ca_cert_path;
+	else if (g_str_equal(key, "WiFi.ClientCertFile") == TRUE)
+		return network->wifi.client_cert_path;
+	else if (g_str_equal(key, "WiFi.PrivateKeyFile") == TRUE)
+		return network->wifi.private_key_path;
+	else if (g_str_equal(key, "WiFi.PrivateKeyPassphrase") == TRUE)
+		return network->wifi.private_key_passphrase;
+	else if (g_str_equal(key, "WiFi.Phase2") == TRUE)
+		return network->wifi.phase2_auth;
 
 	return connman_element_get_string(&network->element, key);
 }
@@ -1330,20 +1442,17 @@ static int network_probe(struct connman_element *element)
 		return err;
 	}
 
-	network->secondary = connman_device_get_secondary(network->device);
-
 	switch (network->type) {
 	case CONNMAN_NETWORK_TYPE_UNKNOWN:
 	case CONNMAN_NETWORK_TYPE_VENDOR:
 		break;
+	case CONNMAN_NETWORK_TYPE_ETHERNET:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 	case CONNMAN_NETWORK_TYPE_CELLULAR:
-	case CONNMAN_NETWORK_TYPE_MBM:
-	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
-		if (network->group != NULL && network->secondary == FALSE)
+		if (network->group != NULL)
 			__connman_profile_add_network(network);
 		break;
 	}
@@ -1367,14 +1476,13 @@ static void network_remove(struct connman_element *element)
 	case CONNMAN_NETWORK_TYPE_UNKNOWN:
 	case CONNMAN_NETWORK_TYPE_VENDOR:
 		break;
+	case CONNMAN_NETWORK_TYPE_ETHERNET:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 	case CONNMAN_NETWORK_TYPE_CELLULAR:
-	case CONNMAN_NETWORK_TYPE_MBM:
-	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
-		if (network->group != NULL && network->secondary == FALSE) {
+		if (network->group != NULL) {
 			__connman_profile_remove_network(network);
 
 			g_free(network->group);
