@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -27,29 +27,12 @@
 #include <stdlib.h>
 #include <sys/types.h>
 
-#define LIBUDEV_I_KNOW_THE_API_IS_SUBJECT_TO_CHANGE
 #include <libudev.h>
 
 #include <glib.h>
 
 #include "connman.h"
 
-#ifdef NEED_UDEV_MONITOR_FILTER
-static int udev_monitor_filter_add_match_subsystem_devtype(struct udev_monitor *udev_monitor,
-				const char *subsystem, const char *devtype)
-{
-	return -EINVAL;
-}
-static int udev_monitor_filter_update(struct udev_monitor *udev_monitor)
-{
-	return -EINVAL;
-}
-static int udev_monitor_filter_remove(struct udev_monitor *udev_monitor)
-{
-	return -EINVAL;
-}
-#endif
-
 static gboolean rfkill_processing = FALSE;
 
 static GSList *device_list = NULL;
@@ -71,6 +54,30 @@ static struct connman_device *find_device(int index)
 	return NULL;
 }
 
+static enum connman_device_type string2devtype(const char *devtype)
+{
+	if (g_strcmp0(devtype, "wlan") == 0)
+		return CONNMAN_DEVICE_TYPE_WIFI;
+	else if (g_strcmp0(devtype, "wimax") == 0)
+		return CONNMAN_DEVICE_TYPE_WIMAX;
+	else if (g_strcmp0(devtype, "wwan") == 0)
+		return CONNMAN_DEVICE_TYPE_CELLULAR;
+
+	return CONNMAN_DEVICE_TYPE_UNKNOWN;
+}
+
+static enum connman_device_type get_device_type(
+		struct udev_device *udev_device, int index)
+{
+	const char *devtype;
+
+	devtype = udev_device_get_devtype(udev_device);
+	if (devtype == NULL)
+		return __connman_inet_get_device_type(index);
+
+	return string2devtype(devtype);
+}
+
 static void add_net_device(struct udev_device *udev_device)
 {
 	struct udev_list_entry *entry;
@@ -101,22 +108,20 @@ static void add_net_device(struct udev_device *udev_device)
 	if (index < 0)
 		return;
 
-	devtype = __connman_inet_get_device_type(index);
+	devtype = get_device_type(udev_device, index);
+
+	DBG("devtype %d", devtype);
 
 	switch (devtype) {
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
 	case CONNMAN_DEVICE_TYPE_VENDOR:
 	case CONNMAN_DEVICE_TYPE_WIMAX:
 	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
 	case CONNMAN_DEVICE_TYPE_GPS:
-	case CONNMAN_DEVICE_TYPE_NOZOMI:
-	case CONNMAN_DEVICE_TYPE_HUAWEI:
-	case CONNMAN_DEVICE_TYPE_NOVATEL:
 		return;
 	case CONNMAN_DEVICE_TYPE_ETHERNET:
 	case CONNMAN_DEVICE_TYPE_WIFI:
-	case CONNMAN_DEVICE_TYPE_MBM:
-	case CONNMAN_DEVICE_TYPE_HSO:
 		break;
 	}
 
@@ -174,6 +179,56 @@ static void remove_net_device(struct udev_device *udev_device)
 	connman_device_unref(device);
 }
 
+static GSList *rfkill_list = NULL;
+
+struct rfkill_data {
+	int phyindex;
+	connman_bool_t blocked;
+};
+
+connman_bool_t __connman_udev_get_blocked(int phyindex)
+{
+	GSList *list;
+
+	if (phyindex < 0)
+		return FALSE;
+
+	for (list = rfkill_list; list; list = rfkill_list->next) {
+		struct rfkill_data *block = list->data;
+
+		if (block->phyindex == phyindex)
+			return block->blocked;
+	}
+
+	return FALSE;
+}
+
+static void update_rfkill_state(int phyindex, connman_bool_t blocked)
+{
+	GSList *list;
+	struct rfkill_data *block;
+
+	DBG("index %d blocked %d", phyindex, blocked);
+
+	for (list = rfkill_list; list; list = rfkill_list->next) {
+		block = list->data;
+
+		if (block->phyindex == phyindex) {
+			block->blocked = blocked;
+			return;
+		}
+	}
+
+	block = g_try_new0(struct rfkill_data, 1);
+	if (block == NULL)
+		return;
+
+	block->phyindex = phyindex;
+	block->blocked = blocked;
+
+	rfkill_list = g_slist_prepend(rfkill_list, block);
+}
+
 static void phyindex_rfkill(int phyindex, connman_bool_t blocked)
 {
 	GSList *list;
@@ -181,6 +236,8 @@ static void phyindex_rfkill(int phyindex, connman_bool_t blocked)
 	if (phyindex < 0)
 		return;
 
+	update_rfkill_state(phyindex, blocked);
+
 	for (list = device_list; list; list = list->next) {
 		struct connman_device *device = list->data;
 
@@ -256,7 +313,7 @@ static void print_properties(struct udev_device *device, const char *prefix)
 				g_str_equal(name, "IFINDEX") == TRUE ||
 				g_str_equal(name, "DEVNAME") == TRUE ||
 				g_str_equal(name, "DEVPATH") == TRUE)
-			connman_debug("%s%s = %s", prefix, name, value);
+			DBG("%s%s = %s", prefix, name, value);
 
 		entry = udev_list_entry_get_next(entry);
 	}
@@ -267,7 +324,7 @@ static void print_device(struct udev_device *device, const char *action)
 	const char *subsystem, *sysname, *driver, *devtype = NULL;
 	struct udev_device *parent;
 
-	connman_debug("=== %s ===", action);
+	DBG("=== %s ===", action);
 	print_properties(device, "");
 
 	parent = udev_device_get_parent(device);
@@ -294,6 +351,9 @@ static void print_device(struct udev_device *device, const char *action)
 	}
 
 	devtype = udev_device_get_devtype(device);
+
+	DBG("devtype %s", devtype);
+
 	sysname = udev_device_get_sysname(device);
 
 	driver = udev_device_get_driver(parent);
@@ -413,62 +473,6 @@ done:
 	return NULL;
 }
 
-char *__connman_udev_get_mbm_devnode(const char *ifname)
-{
-	struct udev_device *device, *parent, *control;
-	const char *driver, *devpath1, *devpath2;
-	char *devnode = NULL;
-	unsigned int i;
-
-	device = udev_device_new_from_subsystem_sysname(udev_ctx,
-							"net", ifname);
-	if (device == NULL)
-		return NULL;
-
-	parent = udev_device_get_parent(device);
-	if (parent == NULL)
-		goto done;
-
-	driver = udev_device_get_driver(parent);
-	if (g_strcmp0(driver, "cdc_ether") != 0)
-		goto done;
-
-	parent = udev_device_get_parent_with_subsystem_devtype(device,
-							"usb", "usb_device");
-	if (parent == NULL)
-		goto done;
-
-	devpath1 = udev_device_get_devpath(parent);
-
-	for (i = 0; i < 64; i++) {
-		char sysname[16];
-
-		snprintf(sysname, sizeof(sysname) - 1, "ttyACM%d", i);
-
-		control = udev_device_new_from_subsystem_sysname(udev_ctx,
-							"tty", sysname);
-		if (control == NULL)
-			continue;
-
-		parent = udev_device_get_parent_with_subsystem_devtype(control,
-							"usb", "usb_device");
-		if (parent == NULL)
-			continue;
-
-		devpath2 = udev_device_get_devpath(parent);
-
-		if (g_strcmp0(devpath1, devpath2) == 0) {
-			devnode = g_strdup(udev_device_get_devnode(control));
-			break;
-		}
-	}
-
-done:
-	udev_device_unref(device);
-
-	return devnode;
-}
-
 void __connman_udev_rfkill(const char *sysname, connman_bool_t blocked)
 {
 	struct udev_device *device, *parent;
@@ -562,6 +566,14 @@ void __connman_udev_cleanup(void)
 	g_slist_free(device_list);
 	device_list = NULL;
 
+	for (list = rfkill_list; list; list = list->next) {
+		struct rfkill_data *block = list->data;
+		g_free(block);
+	}
+
+	g_slist_free(rfkill_list);
+	rfkill_list = NULL;
+
 	if (udev_ctx == NULL)
 		return;
 
